@@ -1,7 +1,7 @@
 /*
- * $Id: cap_flag.c,v 1.2 1997/04/28 00:57:11 morgan Exp $
+ * $Id: cap_flag.c,v 1.2 1997/04/28 00:57:11 morgan Exp morgan $
  *
- * Copyright (c) 1997 Andrew G. Morgan <morgan@parc.power.net>
+ * Copyright (c) 1997-8 Andrew G. Morgan <morgan@linux.kernel.org>
  *
  * See end of file for Log.
  *
@@ -17,17 +17,20 @@
  * the sets stored in cap_d as specified by set and value
  */
 
-int cap_get_flag(cap_t cap_d, cap_value_t value, cap_flag_t set
-		 , cap_flag_value_t *raised)
+int cap_get_flag(cap_t cap_d, cap_value_t value, cap_flag_t set,
+		 cap_flag_value_t *raised)
 {
     /*
      * Do we have a set and a place to store its value?
      * Is it a known capability?
      */
 
-    if (raised && good_cap_t(cap_d) && value >= 0 && value < __CAP_BITS) {
+    if (raised && good_cap_t(cap_d) && value >= 0 && value < __CAP_BITS
+	&& set > 0 && set < NUMBER_OF_CAP_SETS) {
+	__cap_s *cap_p = (__cap_s *) (set*CAP_SET_SIZE
+				      + (__u8 *) &cap_d->set);
 
-	*raised = (cap_d->set[set]._cap_raised(value)) ? CAP_SET:CAP_CLEAR;
+	*raised = isset_cap(cap_p,value) ? CAP_SET:CAP_CLEAR;
 	return 0;
 
     } else {
@@ -43,9 +46,9 @@ int cap_get_flag(cap_t cap_d, cap_value_t value, cap_flag_t set
  * raise/lower a selection of capabilities
  */
 
-int cap_set_flag(cap_t cap_d, cap_flag_t set
-		 , int no_values, cap_value_t *array_values
-		 , cap_flag_value_t raise)
+int cap_set_flag(cap_t cap_d, cap_flag_t set,
+		 int no_values, cap_value_t *array_values,
+		 cap_flag_value_t raise)
 {
     /*
      * Do we have a set and a place to store its value?
@@ -53,16 +56,22 @@ int cap_set_flag(cap_t cap_d, cap_flag_t set
      */
 
     if (good_cap_t(cap_d) && no_values > 0 && no_values <= __CAP_BITS
+	&& (set > 0) && (set < NUMBER_OF_CAP_SETS)
 	&& (raise == CAP_SET || raise == CAP_CLEAR) ) {
 	int i;
-
 	for (i=0; i<no_values; ++i) {
 	    if (array_values[i] < 0 || array_values[i] >= __CAP_BITS) {
 		_cap_debug("weird capability (%d) - skipped", array_values[i]);
-	    } else if (raise == CAP_SET) {
-		cap_d->set[set]._cap_raise(array_values[i]);
-	    } else if (raise == CAP_CLEAR) {
-		cap_d->set[set]._cap_lower(array_values[i]);
+	    } else {
+		int value = array_values[i];
+		__cap_s *cap_p = (__cap_s *) (set*CAP_SET_SIZE
+					      + (__u8 *) &cap_d->set);
+
+		if (raise == CAP_SET) {
+		    cap_p->raise_cap(value);
+		} else {
+		    cap_p->lower_cap(value);
+		}
 	    }
 	}
 	return 0;
@@ -84,7 +93,7 @@ int cap_clear(cap_t cap_d)
 {
     if (good_cap_t(cap_d)) {
 
-	memset(&(cap_d->set), 0, 3*sizeof(__cap_s));
+	memset(&(cap_d->set), 0, sizeof(cap_d->set));
 	return 0;
 
     } else {
