@@ -1,7 +1,7 @@
 /*
- * $Id: cap_proc.c,v 1.4 1997/05/14 05:17:13 morgan Exp $
+ * $Id: cap_proc.c,v 1.4 1997/05/14 05:17:13 morgan Exp morgan $
  *
- * Copyright (c) 1997 Andrew G Morgan <morgan@parc.power.net>
+ * Copyright (c) 1997-8 Andrew G Morgan <morgan@linux.kernel.org>
  *
  * See end of file for Log.
  *
@@ -20,11 +20,9 @@ cap_t cap_get_proc(void)
 	_cap_debug("getting current process' capabilities");
 
 	/* fill the capability sets via a system call */
-	if (_getproccap(sizeof(struct __cap_s),
-			&result->set[CAP_INHERITABLE],
-			&result->set[CAP_PERMITTED],
-			&result->set[CAP_EFFECTIVE] ))
+	if (capget(&result->head, &result->set)) {
 	    cap_free(&result);
+	}
     }
 
     return result;
@@ -38,10 +36,48 @@ int cap_set_proc(cap_t cap_d)
     }
 
     _cap_debug("setting process capabilities");
-    return _setproccap(sizeof(struct __cap_s),
-			  &cap_d->set[CAP_INHERITABLE],
-			  &cap_d->set[CAP_PERMITTED],
-			  &cap_d->set[CAP_EFFECTIVE] );
+    return capset(&cap_d->head, &cap_d->set);
+}
+
+/* the following two functions are not required by POSIX */
+
+/* read the caps on a specific process */
+
+int capgetp(pid_t pid, cap_t cap_d)
+{
+    int error;
+
+    if (!good_cap_t(cap_d)) {
+	errno = EINVAL;
+	return -1;
+    }
+
+    _cap_debug("getting process capabilities for proc %d", pid);
+
+    cap_d->head.pid = pid;
+    error = capget(&cap_d->head, &cap_d->set);
+    cap_d->head.pid = 0;
+
+    return error;
+}
+
+/* set the caps on a specific process/pg etc.. */
+
+int capsetp(pid_t pid, cap_t cap_d)
+{
+    int error;
+
+    if (!good_cap_t(cap_d)) {
+	errno = EINVAL;
+	return -1;
+    }
+
+    _cap_debug("setting process capabilities for proc %d", pid);
+    cap_d->head.pid = pid;
+    error = capset(&cap_d->head, &cap_d->set);
+    cap_d->head.pid = 0;
+
+    return error;
 }
 
 /*
