@@ -1,7 +1,7 @@
 /*
- * $Id: cap_text.c,v 1.3 1997/05/04 05:37:00 morgan Exp $
+ * $Id: cap_text.c,v 1.3 1997/05/04 05:37:00 morgan Exp morgan $
  *
- * Copyright (c) 1997 Andrew G Morgan <morgan@parc.power.net>
+ * Copyright (c) 1997-8 Andrew G Morgan <morgan@linux.kernel.org>
  * Copyright (c) 1997 Andrew Main <zefram@dcs.warwick.ac.uk>
  *
  * See end of file for Log.
@@ -10,11 +10,14 @@
  * representations of capability sets.
  */
 
+#define LIBCAP_PLEASE_INCLUDE_ARRAY
 #include "libcap.h"
 
 #include <ctype.h>
 #include <stdio.h>
 
+char *strdup(const char *s);
+
 /* Maximum output text length (16 per cap) */
 #define CAP_TEXT_SIZE    (16*__CAP_BITS)
 
@@ -27,14 +30,16 @@
  * representation.
  */
 
-static void setbits(__cap_s *a, __cap_s *b)
+#define setbits(A,B) _setbits((__cap_s *)A, (__cap_s *)B)
+static void _setbits(__cap_s *a, __cap_s *b)
 {
     int n;
     for (n = __CAP_BLKS; n--; )
 	a->_blk[n] |= b->_blk[n];
 }
 
-static void clrbits(__cap_s *a, __cap_s *b)
+#define clrbits(A,B) _clrbits((__cap_s *)A, (__cap_s *)B)
+static void _clrbits(__cap_s *a, __cap_s *b)
 {
     int n;
     for (n = __CAP_BLKS; n--; )
@@ -100,9 +105,9 @@ cap_t cap_from_text(const char *str)
 	while (isspace((unsigned char)*str))
 	    str++;
 	if (!*str) {
-	    _cap_debugcap("e = ", &res->set[CAP_EFFECTIVE]);
-	    _cap_debugcap("i = ", &res->set[CAP_INHERITABLE]);
-	    _cap_debugcap("p = ", &res->set[CAP_PERMITTED]);
+	    _cap_debugcap("e = ", &res->set.effective);
+	    _cap_debugcap("i = ", &res->set.inheritable);
+	    _cap_debugcap("p = ", &res->set.permitted);
 	    return res;
 	}
 
@@ -116,7 +121,7 @@ cap_t cap_from_text(const char *str)
 		    n = lookupname(&str);
 		    if (n == -1)
 			goto bad;
-		    list._cap_raise(n);
+		    list.raise_cap(n);
 		}
 		if (*str != ',')
 		    break;
@@ -165,28 +170,28 @@ cap_t cap_from_text(const char *str)
 	    case '=':
 	    case 'P':                                              /* =+ */
 	    case 'M':                                              /* =- */
-		clrbits(&res->set[CAP_EFFECTIVE],   &list);
-		clrbits(&res->set[CAP_INHERITABLE], &list);
-		clrbits(&res->set[CAP_PERMITTED],   &list);
+		clrbits(&res->set.effective,   &list);
+		clrbits(&res->set.inheritable, &list);
+		clrbits(&res->set.permitted,   &list);
 		/* fall through */
 		if (op == 'M')
 		    goto minus;
 	    case '+':
 		if (flags & LIBCAP_EFF)
-		    setbits(&res->set[CAP_EFFECTIVE],   &list);
+		    setbits(&res->set.effective,   &list);
 		if (flags & LIBCAP_INH)
-		    setbits(&res->set[CAP_INHERITABLE], &list);
+		    setbits(&res->set.inheritable, &list);
 		if (flags & LIBCAP_PER)
-		    setbits(&res->set[CAP_PERMITTED],   &list);
+		    setbits(&res->set.permitted,   &list);
 		break;
 	    case '-':
 	    minus:
 	        if (flags & LIBCAP_EFF)
-		    clrbits(&res->set[CAP_EFFECTIVE],   &list);
+		    clrbits(&res->set.effective,   &list);
 		if (flags & LIBCAP_INH)
-		    clrbits(&res->set[CAP_INHERITABLE], &list);
+		    clrbits(&res->set.inheritable, &list);
 		if (flags & LIBCAP_PER)
-		    clrbits(&res->set[CAP_PERMITTED],   &list);
+		    clrbits(&res->set.permitted,   &list);
 		break;
 	    }
 
@@ -221,11 +226,11 @@ static int getstateflags(cap_t caps, int capno)
 {
     int f = 0;
 
-    if (caps->set[CAP_EFFECTIVE]._cap_raised(capno))
+    if (isset_cap((__cap_s *)(&caps->set.effective),capno))
 	f |= LIBCAP_EFF;
-    if (caps->set[CAP_INHERITABLE]._cap_raised(capno))
+    if (isset_cap((__cap_s *)(&caps->set.inheritable),capno))
 	f |= LIBCAP_INH;
-    if (caps->set[CAP_PERMITTED]._cap_raised(capno))
+    if (isset_cap((__cap_s *)(&caps->set.permitted),capno))
 	f |= LIBCAP_PER;
 
     return f;
@@ -246,9 +251,9 @@ char *cap_to_text(cap_t caps, ssize_t *length_p)
 	return NULL;
     }
 
-    _cap_debugcap("e = ", &caps->set[CAP_EFFECTIVE]);
-    _cap_debugcap("i = ", &caps->set[CAP_INHERITABLE]);
-    _cap_debugcap("p = ", &caps->set[CAP_PERMITTED]);
+    _cap_debugcap("e = ", &caps->set.effective);
+    _cap_debugcap("i = ", &caps->set.inheritable);
+    _cap_debugcap("p = ", &caps->set.permitted);
 
     for (n = __CAP_BITS; n--; )
 	histo[getstateflags(caps, n)]++;
@@ -298,7 +303,7 @@ char *cap_to_text(cap_t caps, ssize_t *length_p)
 
     _cap_debug("%s", buf);
     *length_p = p - buf;
-    return buf;
+    return (strdup(buf));
 }
 
 /*
